>From a04a6bf9b442115c03d3dd097d7675f6f3157a2a Mon Sep 17 00:00:00 2001
From: Joseph Tulou <brezhoneg1@yahoo.fr>
Date: Sun, 4 Jan 2009 16:31:08 +0100
Subject: [PATCH] new generic still image demuxer (implemented as a submodule of fake access_demux)

---
 modules/access/fake.c |  117 ++++++++++++++++++++++++++++++++++++++++++-------
 src/input/demux.c     |   20 ++++++++
 2 files changed, 121 insertions(+), 16 deletions(-)

diff --git a/modules/access/fake.c b/modules/access/fake.c
index 2e0d990..32fd155 100644
--- a/modules/access/fake.c
+++ b/modules/access/fake.c
@@ -33,11 +33,13 @@
 #include <vlc_plugin.h>
 #include <vlc_access.h>
 #include <vlc_demux.h>
+#include <vlc_image.h>
 
 /*****************************************************************************
  * Module descriptior
  *****************************************************************************/
 static int  Open ( vlc_object_t * );
+static int  ImageOpen ( vlc_object_t * );
 static void Close( vlc_object_t * );
 
 #define CACHING_TEXT N_("Caching value in ms")
@@ -72,11 +74,19 @@ vlc_module_begin ()
     add_shortcut( "fake" )
     set_capability( "access_demux", 0 )
     set_callbacks( Open, Close )
+
+    add_submodule ()
+    set_description( N_("Still image generic demux") )
+    add_shortcut( "image" )
+    set_capability( "demux", 0 )
+    set_callbacks( ImageOpen, Close )
+
 vlc_module_end ()
 
 /*****************************************************************************
  * Access: local prototypes
  *****************************************************************************/
+static int fakeOpen ( demux_t *, vlc_fourcc_t );
 static int Demux  ( demux_t * );
 static int Control( demux_t *, int, va_list );
 
@@ -85,6 +95,10 @@ struct demux_sys_t
     float f_fps;
     mtime_t i_last_pts, i_duration, i_first_pts, i_end_pts, i_pause_pts;
 
+    vlc_fourcc_t  i_codec;
+    int           i_size;
+    char          *p_image;
+
     es_out_id_t  *p_es_video;
 };
 
@@ -94,25 +108,94 @@ struct demux_sys_t
 static int Open( vlc_object_t *p_this )
 {
     demux_t     *p_demux = (demux_t*)p_this;
-    demux_sys_t *p_sys;
-    es_format_t fmt;
 
     /* Only when selected */
     if( *p_demux->psz_access == '\0' )
         return VLC_EGENERIC;
 
+    return fakeOpen( p_demux, VLC_FOURCC('f','a','k','e') );
+}
+
+/*****************************************************************************
+ * Close: close device, free resources
+ *****************************************************************************/
+static void Close( vlc_object_t *p_this )
+{
+    demux_t     *p_demux = (demux_t *)p_this;
+    demux_sys_t *p_sys   = p_demux->p_sys;
+
+    if( p_sys->p_image )
+        free( p_sys->p_image );
+    free( p_sys );
+}
+
+/*****************************************************************************
+ * ImageOpen: opens a generic image demux
+ *****************************************************************************/
+static int ImageOpen( vlc_object_t *p_this )
+{
+    demux_t     *p_demux = (demux_t*)p_this;
+    demux_sys_t *p_sys;
+
+    if( !p_demux->psz_path || !*(p_demux->psz_path) )
+        return VLC_EGENERIC;
+
+    vlc_fourcc_t i_codec = image_Ext2Fourcc( p_demux->psz_path );
+    if( !i_codec )
+        return VLC_EGENERIC;
+
+    char* p_codec = (char*) &i_codec;
+    msg_Dbg( p_demux, "still image detected with codec format %c%c%c%c",
+               p_codec[0], p_codec[1], p_codec[2], p_codec[3] ); 
+
+    int ret = fakeOpen( p_demux, i_codec );
+    if( ret != VLC_SUCCESS )
+        return ret;
+
+    /* keep local copy of still image to be used later on */
+
+    p_sys = p_demux->p_sys;
+
+    p_sys->i_size = stream_Size( p_demux->s );
+    if( !p_sys->i_size )
+    {
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+
+    p_sys->p_image = malloc( p_sys->i_size );
+    if( !p_sys->p_image )
+    {
+        free( p_sys );
+        return VLC_ENOMEM;
+    }
+    stream_Read( p_demux->s, p_sys->p_image, p_sys->i_size );
+
+    return VLC_SUCCESS;
+}
+
+/*****************************************************************************
+ * fakeOpen : set up a fake demux for both capabilities
+ *****************************************************************************/
+static int fakeOpen( demux_t *p_demux, vlc_fourcc_t i_codec )
+{
+    demux_sys_t *p_sys;
+    es_format_t fmt;
+
     /* Set up p_demux */
     DEMUX_INIT_COMMON(); p_sys = p_demux->p_sys;
     p_demux->info.i_update = 0;
     p_demux->info.i_title = 0;
     p_demux->info.i_seekpoint = 0;
 
+    p_sys->i_codec = i_codec;
+
     p_sys->i_duration =
         (mtime_t)var_CreateGetInteger( p_demux, "fake-duration" ) * 1000;
     p_sys->f_fps = var_CreateGetFloat( p_demux, "fake-fps" );
 
     /* Declare the elementary stream */
-    es_format_Init( &fmt, VIDEO_ES, VLC_FOURCC('f','a','k','e') );
+    es_format_Init( &fmt, VIDEO_ES, i_codec );
     fmt.i_id = var_CreateGetInteger( p_demux, "fake-id" );
     p_sys->p_es_video = es_out_Add( p_demux->out, &fmt );
 
@@ -121,18 +204,6 @@ static int Open( vlc_object_t *p_this )
 
     return VLC_SUCCESS;
 }
-
-/*****************************************************************************
- * Close: close device, free resources
- *****************************************************************************/
-static void Close( vlc_object_t *p_this )
-{
-    demux_t     *p_demux = (demux_t *)p_this;
-    demux_sys_t *p_sys   = p_demux->p_sys;
-
-    free( p_sys );
-}
-
 /*****************************************************************************
  * Control:
  *****************************************************************************/
@@ -228,6 +299,7 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
 static int Demux( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
+    block_t *p_block;
 
     if ( !p_sys->i_last_pts )
     {
@@ -243,7 +315,20 @@ static int Demux( demux_t *p_demux )
         mwait( p_sys->i_last_pts );
     }
 
-    block_t *p_block = block_New( p_demux, 1 );
+    if( p_sys->p_image )
+    {
+        p_block = block_New( p_demux, p_sys->i_size );
+        if( !p_block )
+            return VLC_ENOMEM;
+        memcpy( p_block->p_buffer, p_sys->p_image, p_sys->i_size );
+    }
+    else
+    {
+        p_block = block_New( p_demux, 1 );
+        if( !p_block )
+            return VLC_ENOMEM;
+    }
+
     p_block->i_flags |= BLOCK_FLAG_TYPE_I;
     p_block->i_dts = p_block->i_pts = p_sys->i_last_pts;
 
diff --git a/src/input/demux.c b/src/input/demux.c
index 026ad4b..35eca52 100644
--- a/src/input/demux.c
+++ b/src/input/demux.c
@@ -111,6 +111,26 @@ demux_t *__demux_New( vlc_object_t *p_obj,
             { "voc",  "voc" },
             { "mid",  "smf" },
             { "rmi",  "smf" },
+
+            { "jpeg", "image" },
+            { "jpg",  "image" },
+            { "ljpg", "image" },
+            { "png",  "image" },
+            { "pgm",  "image" },
+            { "pgmyuv", "image" },
+            { "pbm",  "image" },
+            { "pam",  "image" },
+            { "tga",  "image" },
+            { "bmp",  "image" },
+            { "pnm",  "image" },
+            { "xpm",  "image" },
+            { "xcf",  "image" },
+            { "pcx",  "image" },
+            { "gif",  "image" },
+            { "tif",  "image" },
+            { "tiff", "image" },
+            { "lbm",  "image" },
+
             { "",  "" },
         };
         /* Here, we don't mind if it does not work, it must be quick */
-- 
1.5.2.5

